##############################################################################
#
# Copyright (c) 2006-2009 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Chameleon page template components"""

import os
import sys
import martian

from grokcore.component import GlobalUtility, implements, name
from grokcore.view import interfaces
from grokcore.view.components import GrokTemplate
from chameleon.zpt.template import PageTemplate, PageTemplateFile
from chameleon.tales import PythonExpr
from chameleon.tales import StringExpr
from chameleon.tales import NotExpr
from chameleon.tales import ExistsExpr
from chameleon.tales import ImportExpr
from chameleon.tales import StructureExpr
from z3c.pt.expressions import PathExpr, ProviderExpr


#
# Chameleon Zope Page Templates...
#
class PageTemplate(PageTemplate, GrokTemplate):
    """A `z3c.pt` page template suitable for use with views.

    This page template implementation is different from `z3c.pt`
    implementation in two respects:

    - It sets ``python`` as default mode (instead of ``path``)
    - It injects any views ``static`` variable in template namespace.
    """
    default_expression = 'python'  # Use the chameleon default

    expression_types = {
        'python': PythonExpr,
        'string': StringExpr,
        'not': NotExpr,
        'path': PathExpr,
        'provider': ProviderExpr,
        'exists': ExistsExpr,
        'import': ImportExpr,
        'structure': StructureExpr,
        }


class PageTemplateFile(PageTemplate, PageTemplateFile):
    """A `z3c.pt` page template file suitable for use with views.

    This implementation is different from `z3c.pt` implementation in
    two respects:

    - It sets ``python`` as default mode (instead of ``path``)
    - It injects any views ``static`` variable in template namespace.
    """
    default_expression = 'python'


class ChameleonPageTemplate(GrokTemplate):

    def setFromString(self, string):
        self._filename = None
        self._template = PageTemplate(string)

    def setFromFilename(self, filename, _prefix=None):
        self._filename = filename
        self._prefix = _prefix
        self._template = PageTemplateFile(os.path.join(_prefix, filename))
        return

    @property
    def macros(self):
        return self._template.macros

    def render(self, view):
        return self._template(**self.getNamespace(view))


class ChameleonPageTemplateFile(ChameleonPageTemplate):

    def __init__(self, filename, _prefix=None):
        self.__grok_module__ = martian.util.caller_module()
        if _prefix is None:
            module = sys.modules[self.__grok_module__]
            _prefix = os.path.dirname(module.__file__)
        self.setFromFilename(filename, _prefix)


class ChameleonPageTemplateFactory(GlobalUtility):
    implements(interfaces.ITemplateFileFactory)
    name('cpt')

    def __call__(self, filename, _prefix=None):
        return ChameleonPageTemplate(filename=filename, _prefix=_prefix)


from grokcore.component.interfaces import IContext
from zope.container.btree import BTreeContainer
from zope.interface import implements
from grokcore.chameleon import components

import grokcore.view
import grokcore.viewlet

class Mammoth(BTreeContainer):
    implements(IContext)

class CavePainting(grokcore.view.View):
    pass

class Food(grokcore.view.View):

    text = "<ME GROK EAT MAMMOTH!>"

    def me_do(self):
        return self.text

class Inline(grokcore.view.View):
    sometext = 'Some Text'

inline = components.ChameleonPageTemplate(
    "<html><body>ME GROK HAS INLINES! ${view.sometext}</body></html>")

class Vars(grokcore.view.View):
    pass

class Expressions(grokcore.view.View):
    pass

class MainArea(grokcore.viewlet.ViewletManager):
    grokcore.viewlet.name('main')

class MainContent(grokcore.viewlet.Viewlet):
    grokcore.viewlet.view(Expressions)
    grokcore.viewlet.viewletmanager(MainArea)
    def render(self):
        return 'Hello from viewlet'

class MacroMaster(grokcore.view.View):
    """A view with a template that contains macro defs.
    """
    pass

class MacroUser(grokcore.view.View):
    """A view with a template that uses macros.
    """
    pass

class Namespace(grokcore.view.View):

    def namespace(self):
        return {'myname': 'Henk'}



"""Test setup for grokcore.chameleon.
"""
import doctest
import unittest
import zope.interface
import zope.component
import grokcore.chameleon

from zope.traversing.interfaces import ITraversable
from zope.traversing.browser.interfaces import IAbsoluteURL
from zope.publisher.interfaces.browser import IBrowserRequest
from grokcore.chameleon.tests import FunctionalLayer

FLAGS = (doctest.ELLIPSIS | doctest.NORMALIZE_WHITESPACE)

class DummyResource(object):
    """ Dummy resource implementation. """
    zope.interface.implements(ITraversable, IAbsoluteURL)

    def __init__(self, request, name=''):
        self.request = request
        self.name = name

    def traverse(self, name, furtherPath):
        name = '%s/%s' % (self.name, name)
        return DummyResource(self.request, name=name)

    def __str__(self):
        return 'dummy:%s' % self.name

def setUpStatic(test):
    # Register a dummy resource for static folder.
    zope.component.provideAdapter(
        factory=DummyResource,
        adapts=(IBrowserRequest,),
        provides=zope.interface.Interface,
        name='grokcore.chameleon.tests.cpt_fixture'
        )
    return


def test_suite():
    """Get a testsuite of all doctests.
    """
    suite = unittest.TestSuite()
    test = doctest.DocFileSuite(
        'README.txt',
        setUp=setUpStatic,
        package=grokcore.chameleon,
        globs=dict(
            getRootFolder=FunctionalLayer.getRootFolder,
            ),
        optionflags=FLAGS,
        )
    test.layer = FunctionalLayer
    suite.addTest(test)
    return suite



##############################################################################
#
# Copyright (c) 2006-2010 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
# $Id: __init__.py 122220 2011-07-14 13:26:32Z janwijbrand $ $Rev$ $Author$ $Date$
"""A functional test layer.
"""
import grokcore.chameleon
from zope.app.wsgi.testlayer import BrowserLayer

FunctionalLayer = BrowserLayer(grokcore.chameleon, 'ftesting.zcml')



import os
from setuptools import setup, find_packages

version = '1.0rc4'

install_requires = [
    'setuptools',
    'grokcore.component',
    'grokcore.view',
    'Chameleon >= 2.0-rc1',
    'z3c.pt >= 2.0-rc1',
    ]

tests_require = [
    'grokcore.viewlet',
    'zope.app.wsgi',
    'zope.app.appsetup',
    ]

long_description = (open('README.txt').read() +
                    '\n\n' +
                    open(os.path.join('src', 'grokcore', 'chameleon',
                                      'README.txt')).read() +
                    '\n\n' +
                    open('CHANGES.txt').read())

setup(name='grokcore.chameleon',
      version=version,
      description="Chameleon page template support for Grok",
      long_description=long_description,
      # Get strings from http://www.python.org/pypi?%3Aaction=list_classifiers
      classifiers=['Development Status :: 3 - Alpha',
                   'Environment :: Web Environment',
                   'Intended Audience :: Developers',
                   'License :: OSI Approved :: Zope Public License',
                   'Programming Language :: Python :: 2.5',
                   'Operating System :: OS Independent',
                   'Topic :: Internet :: WWW/HTTP',
                   ],
      keywords="grok chameleon template",
      author="Uli Fouquet",
      author_email="grok-dev@zope.org",
      url="http://pypi.python.org/pypi/grokcore.chameleon",
      license="ZPL",
      package_dir={'': 'src'},
      packages=find_packages('src'),
      namespace_packages=['grokcore'],
      include_package_data=True,
      zip_safe=False,
      install_requires=install_requires,
      tests_require=tests_require,
      extras_require={'test': tests_require},
      )
